@@ -21,12 +21,12 @@ from MoinMoin import log
 logging = log.getLogger(__name__)
 
 from MoinMoin import config
-from MoinMoin.util import pysupport, lock
 from MoinMoin.support.python_compatibility import rsplit
 from inspect import getargspec, isfunction, isclass, ismethod
 
-from MoinMoin import web # needed so that next line works:
+from MoinMoin import web # needed so that next lines work:
 import werkzeug
+from MoinMoin.util import pysupport, lock
 
 # Exceptions
 class InvalidFileNameError(Exception):
@@ -873,6 +873,13 @@ MIMETYPES_MORE = {
  '.swf': 'application/x-shockwave-flash',
  '.moin': 'text/moin-wiki',
  '.creole': 'text/creole',
+ # Windows Server 2003 / Python 2.7 has no or strange entries for these:
+ '.svg': 'image/svg+xml',
+ '.svgz': 'image/svg+xml',
+ '.png': 'image/png',
+ '.jpg': 'image/jpeg',
+ '.jpeg': 'image/jpeg',
+ '.gif': 'image/gif',
 }
 
 # add all mimetype patterns of pygments
